@@ -430,6 +430,29 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
   gst_ffmpeg_caps_with_codecid (oclass->in_plugin->id,
       oclass->in_plugin->type, allowed_caps, ffmpegenc->context);
 
+  /* open codec */
+  if (gst_ffmpeg_avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0) {
+    gst_caps_unref (allowed_caps);
+    goto open_codec_fail;
+  }
+
+  /* is the colourspace correct? */
+  if (pix_fmt != ffmpegenc->context->pix_fmt) {
+    gst_caps_unref (allowed_caps);
+    goto pix_fmt_err;
+  }
+
+  /* we may have failed mapping caps to a pixfmt,
+   * and quite some codecs do not make up their own mind about that
+   * in any case, _NONE can never work out later on */
+  if (pix_fmt == AV_PIX_FMT_NONE) {
+    gst_caps_unref (allowed_caps);
+    goto bad_input_fmt;
+  }
+
+  /* second pass stats buffer no longer needed */
+  g_free (ffmpegenc->context->stats_in);
+
   /* try to set this caps on the other side */
   other_caps = gst_ffmpeg_codecid_to_caps (oclass->in_plugin->id,
       ffmpegenc->context, TRUE);
@@ -450,23 +473,6 @@ gst_ffmpegvidenc_set_format (GstVideoEncoder * encoder,
 
   GST_DEBUG_OBJECT (ffmpegenc, "codec flags 0x%08x", ffmpegenc->context->flags);
 
-  /* open codec */
-  if (gst_ffmpeg_avcodec_open (ffmpegenc->context, oclass->in_plugin) < 0)
-    goto open_codec_fail;
-
-  /* is the colourspace correct? */
-  if (pix_fmt != ffmpegenc->context->pix_fmt)
-    goto pix_fmt_err;
-
-  /* we may have failed mapping caps to a pixfmt,
-   * and quite some codecs do not make up their own mind about that
-   * in any case, _NONE can never work out later on */
-  if (pix_fmt == AV_PIX_FMT_NONE)
-    goto bad_input_fmt;
-
-  /* second pass stats buffer no longer needed */
-  g_free (ffmpegenc->context->stats_in);
-
   /* Store input state and set output state */
   if (ffmpegenc->input_state)
     gst_video_codec_state_unref (ffmpegenc->input_state);
@@ -952,8 +958,16 @@ gst_ffmpegvidenc_register (GstPlugin * plugin)
     if (in_plugin->id == AV_CODEC_ID_RAWVIDEO ||
         in_plugin->id == AV_CODEC_ID_V210 ||
         in_plugin->id == AV_CODEC_ID_V210X ||
+        in_plugin->id == AV_CODEC_ID_V308 ||
+        in_plugin->id == AV_CODEC_ID_V408 ||
+        in_plugin->id == AV_CODEC_ID_V410 ||
         in_plugin->id == AV_CODEC_ID_R210
-        || in_plugin->id == AV_CODEC_ID_ZLIB) {
+        || in_plugin->id == AV_CODEC_ID_AYUV
+        || in_plugin->id == AV_CODEC_ID_Y41P
+        || in_plugin->id == AV_CODEC_ID_012V
+        || in_plugin->id == AV_CODEC_ID_YUV4
+        || in_plugin->id == AV_CODEC_ID_ZLIB
+        || in_plugin->id == AV_CODEC_ID_WRAPPED_AVFRAME) {
       goto next;
     }
 
@@ -967,6 +981,20 @@ gst_ffmpegvidenc_register (GstPlugin * plugin)
       goto next;
     }
 
+    if (strstr (in_plugin->name, "vaapi")) {
+      GST_DEBUG
+          ("Ignoring VAAPI encoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      goto next;
+    }
+
+    if (g_str_has_suffix (in_plugin->name, "_nvenc")) {
+      GST_DEBUG
+          ("Ignoring nvenc encoder %s. We can't handle this outside of ffmpeg",
+          in_plugin->name);
+      goto next;
+    }
+
     /* only video encoders */
     if (!av_codec_is_encoder (in_plugin)
         || in_plugin->type != AVMEDIA_TYPE_VIDEO)
