@@ -658,7 +658,8 @@ static int rate_control(AVCodecContext *avctx, void *arg)
     VC2EncContext *s = slice_dat->ctx;
     const int sx = slice_dat->x;
     const int sy = slice_dat->y;
-    int quant_buf[2], bits_buf[2], quant = s->q_start, range = s->q_start/3;
+    int bits_last = INT_MAX, quant_buf[2] = {-1, -1};
+    int quant = s->q_start, range = s->q_start/3;
     const int64_t top = slice_dat->bits_ceil;
     const double percent = s->tolerance;
     const double bottom = top - top*(percent/100.0f);
@@ -670,14 +671,13 @@ static int rate_control(AVCodecContext *avctx, void *arg)
         bits = count_hq_slice(s, sx, sy, quant);
         range = av_clip(range/2, 1, s->q_ceil);
         if (quant_buf[1] == quant) {
-            quant = bits_buf[0] < bits ? quant_buf[0] : quant;
-            bits = bits_buf[0] < bits ? bits_buf[0] : bits;
+            quant = bits_last < bits ? quant_buf[0] : quant;
+            bits  = bits_last < bits ? bits_last : bits;
             break;
         }
         quant_buf[1] = quant_buf[0];
         quant_buf[0] = quant;
-        bits_buf[1] = bits_buf[0];
-        bits_buf[0] = bits;
+        bits_last = bits;
     }
     slice_dat->quant_idx = av_clip(quant, 0, s->q_ceil);
     slice_dat->bytes = FFALIGN((bits >> 3), s->size_scaler) + 4 + s->prefix_bytes;
@@ -867,7 +867,7 @@ static int dwt_plane(AVCodecContext *avctx, void *arg)
         }
     }
 
-    memset(buf, 0, (p->coef_stride*p->dwt_height - p->height*p->width)*sizeof(dwtcoef));
+    memset(buf, 0, p->coef_stride * (p->dwt_height - p->height) * sizeof(dwtcoef));
 
     for (level = s->wavelet_depth-1; level >= 0; level--) {
         const SubBand *b = &p->band[level][0];
